@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Mikael Hallendal <micke@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
@@ -33,6 +33,9 @@
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-enum-types.h>
+#include <libempathy/empathy-contact-manager.h>
+
 #include "empathy-contact-list-store.h"
 #include "empathy-ui-utils.h"
 #include "empathy-gtk-enum-types.h"
@@ -60,6 +63,8 @@ typedef struct {
 	gboolean                    show_active;
 	EmpathyContactListStoreSort sort_criterium;
 	guint                       inhibit_active;
+	guint                       setup_idle_id;
+	gboolean                    dispose_has_run;
 } EmpathyContactListStorePriv;
 
 typedef struct {
@@ -80,7 +85,7 @@ typedef struct {
 	gboolean                remove;
 } ShowActiveData;
 
-static void             contact_list_store_finalize                  (GObject                       *object);
+static void             contact_list_store_dispose                  (GObject                       *object);
 static void             contact_list_store_get_property              (GObject                       *object,
 								      guint                          param_id,
 								      GValue                        *value,
@@ -190,6 +195,7 @@ contact_list_store_iface_setup (gpointer user_data)
 	}
 	g_list_free (contacts);
 
+	priv->setup_idle_id = 0;
 	return FALSE;
 }
 
@@ -203,7 +209,7 @@ contact_list_store_set_contact_list (EmpathyContactListStore *store,
 	priv->list = g_object_ref (list_iface);
 
 	/* Let a chance to have all properties set before populating */
-	g_idle_add (contact_list_store_iface_setup, store);
+	priv->setup_idle_id = g_idle_add (contact_list_store_iface_setup, store);
 }
 
 static void
@@ -211,7 +217,7 @@ empathy_contact_list_store_class_init (EmpathyContactListStoreClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	object_class->finalize = contact_list_store_finalize;
+	object_class->dispose = contact_list_store_dispose;
 	object_class->get_property = contact_list_store_get_property;
 	object_class->set_property = contact_list_store_set_property;
 
@@ -283,11 +289,15 @@ empathy_contact_list_store_init (EmpathyContactListStore *store)
 }
 
 static void
-contact_list_store_finalize (GObject *object)
+contact_list_store_dispose (GObject *object)
 {
 	EmpathyContactListStorePriv *priv = GET_PRIV (object);
 	GList                       *contacts, *l;
 
+	if (priv->dispose_has_run)
+		return;
+	priv->dispose_has_run = TRUE;
+
 	contacts = empathy_contact_list_get_members (priv->list);
 	for (l = contacts; l; l = l->next) {
 		g_signal_handlers_disconnect_by_func (l->data,
@@ -310,7 +320,11 @@ contact_list_store_finalize (GObject *object)
 		g_source_remove (priv->inhibit_active);
 	}
 
-	G_OBJECT_CLASS (empathy_contact_list_store_parent_class)->finalize (object);
+	if (priv->setup_idle_id != 0) {
+		g_source_remove (priv->setup_idle_id);
+	}
+
+	G_OBJECT_CLASS (empathy_contact_list_store_parent_class)->dispose (object);
 }
 
 static void
@@ -601,7 +615,7 @@ empathy_contact_list_store_set_sort_criterium (EmpathyContactListStore     *stor
 						      EMPATHY_CONTACT_LIST_STORE_COL_STATUS,
 						      GTK_SORT_ASCENDING);
 		break;
-		
+
 	case EMPATHY_CONTACT_LIST_STORE_SORT_NAME:
 		gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
 						      EMPATHY_CONTACT_LIST_STORE_COL_NAME,
@@ -707,7 +721,7 @@ empathy_contact_list_store_search_equal_func (GtkTreeModel *model,
 	name_folded = g_utf8_casefold (name, -1);
 	key_folded = g_utf8_casefold (key, -1);
 
-	if (name_folded && key_folded && 
+	if (name_folded && key_folded &&
 	    strstr (name_folded, key_folded)) {
 		ret = FALSE;
 	} else {
@@ -725,19 +739,23 @@ static void
 contact_list_store_setup (EmpathyContactListStore *store)
 {
 	EmpathyContactListStorePriv *priv;
-	GType                       types[] = {G_TYPE_STRING,        /* Status icon-name */
-					       GDK_TYPE_PIXBUF,      /* Avatar pixbuf */
-					       G_TYPE_BOOLEAN,       /* Avatar pixbuf visible */
-					       G_TYPE_STRING,        /* Name */
-					       G_TYPE_STRING,        /* Status string */
-					       G_TYPE_BOOLEAN,       /* Show status */
-					       EMPATHY_TYPE_CONTACT, /* Contact type */
-					       G_TYPE_BOOLEAN,       /* Is group */
-					       G_TYPE_BOOLEAN,       /* Is active */
-					       G_TYPE_BOOLEAN,       /* Is online */
-					       G_TYPE_BOOLEAN,       /* Is separator */
-					       G_TYPE_BOOLEAN};      /* Can VoIP */
-	
+	GType types[] = {
+		G_TYPE_STRING,        /* Status icon-name */
+		GDK_TYPE_PIXBUF,      /* Avatar pixbuf */
+		G_TYPE_BOOLEAN,       /* Avatar pixbuf visible */
+		G_TYPE_STRING,        /* Name */
+		G_TYPE_STRING,        /* Status string */
+		G_TYPE_BOOLEAN,       /* Show status */
+		EMPATHY_TYPE_CONTACT, /* Contact type */
+		G_TYPE_BOOLEAN,       /* Is group */
+		G_TYPE_BOOLEAN,       /* Is active */
+		G_TYPE_BOOLEAN,       /* Is online */
+		G_TYPE_BOOLEAN,       /* Is separator */
+		G_TYPE_BOOLEAN,       /* Can make audio calls */
+		G_TYPE_BOOLEAN,       /* Can make video calls */
+		EMPATHY_TYPE_CONTACT_LIST_FLAGS, /* Flags */
+	};
+
 	priv = GET_PRIV (store);
 
 	gtk_tree_store_set_column_types (GTK_TREE_STORE (store),
@@ -850,9 +868,11 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 	EmpathyContactListStorePriv *priv;
 	GtkTreeIter                 iter;
 	GList                      *groups = NULL, *l;
+	TpConnection               *connection;
+	EmpathyContactListFlags     flags = 0;
 
 	priv = GET_PRIV (store);
-	
+
 	if (EMP_STR_EMPTY (empathy_contact_get_name (contact)) ||
 	    (!priv->show_offline && !empathy_contact_is_online (contact))) {
 		return;
@@ -862,15 +882,44 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 		groups = empathy_contact_list_get_groups (priv->list, contact);
 	}
 
+	connection = empathy_contact_get_connection (contact);
+	if (EMPATHY_IS_CONTACT_MANAGER (priv->list)) {
+		flags = empathy_contact_manager_get_flags_for_connection (
+			EMPATHY_CONTACT_MANAGER (priv->list), connection);
+	}
 	/* If no groups just add it at the top level. */
 	if (!groups) {
+		GtkTreeIter iter;
+		GtkTreeModel *model = GTK_TREE_MODEL (store);
+
+		if (gtk_tree_model_get_iter_first (model, &iter)) do {
+			EmpathyContact *c;
+
+			gtk_tree_model_get (model, &iter,
+				EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &c,
+				-1);
+
+			if (c == contact) {
+				g_object_unref (c);
+				return;
+			}
+			if (c != NULL)
+				g_object_unref (c);
+		} while (gtk_tree_model_iter_next (model, &iter));
+
 		gtk_tree_store_append (GTK_TREE_STORE (store), &iter, NULL);
 		gtk_tree_store_set (GTK_TREE_STORE (store), &iter,
 				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
 				    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VOIP, empathy_contact_can_voip (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
+				      empathy_contact_get_capabilities (contact) &
+				        EMPATHY_CAPABILITIES_AUDIO,
+				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
+				      empathy_contact_get_capabilities (contact) &
+				        EMPATHY_CAPABILITIES_VIDEO,
+				    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
 				    -1);
 	}
 
@@ -887,7 +936,13 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 				    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VOIP, empathy_contact_can_voip (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
+				      empathy_contact_get_capabilities (contact) &
+				        EMPATHY_CAPABILITIES_AUDIO,
+				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
+				      empathy_contact_get_capabilities (contact) &
+				        EMPATHY_CAPABILITIES_VIDEO,
+				    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
 				    -1);
 		g_free (l->data);
 	}
@@ -911,7 +966,7 @@ contact_list_store_remove_contact (EmpathyContactListStore *store,
 	if (!iters) {
 		return;
 	}
-	
+
 	/* Clean up model */
 	model = GTK_TREE_MODEL (store);
 
@@ -920,7 +975,7 @@ contact_list_store_remove_contact (EmpathyContactListStore *store,
 
 		/* NOTE: it is only <= 2 here because we have
 		 * separators after the group name, otherwise it
-		 * should be 1. 
+		 * should be 1.
 		 */
 		if (gtk_tree_model_iter_parent (model, &parent, l->data) &&
 		    gtk_tree_model_iter_n_children (model, &parent) <= 2) {
@@ -1056,7 +1111,12 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_ONLINE, now_online,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VOIP, empathy_contact_can_voip (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
+				      empathy_contact_get_capabilities (contact) &
+				        EMPATHY_CAPABILITIES_AUDIO,
+				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
+				      empathy_contact_get_capabilities (contact) &
+				        EMPATHY_CAPABILITIES_VIDEO,
 				    -1);
 	}
 
@@ -1138,9 +1198,9 @@ contact_list_store_contact_active_new (EmpathyContactListStore *store,
 	ShowActiveData *data;
 
 	DEBUG ("Contact:'%s' now active, and %s be removed",
-		empathy_contact_get_name (contact), 
+		empathy_contact_get_name (contact),
 		remove ? "WILL" : "WILL NOT");
-	
+
 	data = g_slice_new0 (ShowActiveData);
 
 	data->store = g_object_ref (store);
@@ -1259,7 +1319,7 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 		}
 
 		gtk_tree_store_append (GTK_TREE_STORE (store),
-				       &iter_separator, 
+				       &iter_separator,
 				       &iter_group);
 		gtk_tree_store_set (GTK_TREE_STORE (store), &iter_separator,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, TRUE,
@@ -1293,28 +1353,6 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 	}
 }
 
-static guint
-contact_list_store_ordered_presence (McPresence state)
-{
-	switch (state) {
-	case MC_PRESENCE_UNSET:
-	case MC_PRESENCE_OFFLINE:
-		return 5;
-	case MC_PRESENCE_AVAILABLE:
-		return 0;
-	case MC_PRESENCE_AWAY:
-		return 2;
-	case MC_PRESENCE_EXTENDED_AWAY:
-		return 3;
-	case MC_PRESENCE_HIDDEN:
-		return 4;
-	case MC_PRESENCE_DO_NOT_DISTURB:
-		return 1;
-	default:
-		g_return_val_if_reached (6);
-	}
-}
-
 static gint
 contact_list_store_state_sort_func (GtkTreeModel *model,
 				    GtkTreeIter  *iter_a,
@@ -1325,7 +1363,6 @@ contact_list_store_state_sort_func (GtkTreeModel *model,
 	gchar          *name_a, *name_b;
 	gboolean        is_separator_a, is_separator_b;
 	EmpathyContact *contact_a, *contact_b;
-	guint           presence_a, presence_b;
 
 	gtk_tree_model_get (model, iter_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_a,
@@ -1361,16 +1398,11 @@ contact_list_store_state_sort_func (GtkTreeModel *model,
 	/* If we managed to get this far, we can start looking at
 	 * the presences.
 	 */
-	presence_a = empathy_contact_get_presence (EMPATHY_CONTACT (contact_a));
-	presence_a = contact_list_store_ordered_presence (presence_a);
-	presence_b = empathy_contact_get_presence (EMPATHY_CONTACT (contact_b));
-	presence_b = contact_list_store_ordered_presence (presence_b);
+	ret_val = -tp_connection_presence_type_cmp_availability (
+		empathy_contact_get_presence (EMPATHY_CONTACT (contact_a)),
+		empathy_contact_get_presence (EMPATHY_CONTACT (contact_b)));
 
-	if (presence_a < presence_b) {
-		ret_val = -1;
-	} else if (presence_a > presence_b) {
-		ret_val = 1;
-	} else {
+	if (ret_val == 0) {
 		/* Fallback: compare by name */
 		ret_val = g_utf8_collate (name_a, name_b);
 	}
