@@ -19,6 +19,9 @@
  *          Cosimo Cecchi <cosimo.cecchi@collabora.co.uk>
  */
 
+#define DISPATCHER_BUS_NAME TP_CLIENT_BUS_NAME_BASE "Empathy"
+#define DISPATCHER_OBJECT_PATH TP_CLIENT_OBJECT_PATH_BASE "Empathy"
+
 #include <config.h>
 
 #include <string.h>
@@ -31,12 +34,15 @@
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/proxy-subclass.h>
 #include <telepathy-glib/gtypes.h>
-
-#include <libmissioncontrol/mission-control.h>
+#include <telepathy-glib/defs.h>
+#include <telepathy-glib/svc-client.h>
+#include <telepathy-glib/svc-generic.h>
+#include <telepathy-glib/interfaces.h>
 
 #include <extensions/extensions.h>
 
 #include "empathy-dispatcher.h"
+#include "empathy-handler.h"
 #include "empathy-utils.h"
 #include "empathy-tube-handler.h"
 #include "empathy-account-manager.h"
@@ -50,8 +56,9 @@
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyDispatcher)
 typedef struct
 {
+  gboolean dispose_has_run;
+
   EmpathyAccountManager *account_manager;
-  MissionControl *mc;
   /* connection to connection data mapping */
   GHashTable *connections;
   GHashTable *outstanding_classes_requests;
@@ -60,12 +67,40 @@ typedef struct
 
   /* channels which the dispatcher is listening "invalidated" */
   GList *channels;
+  GPtrArray *array;
+
+  /* main handler */
+  EmpathyHandler *handler;
+
+  /* extra handlers */
+  GList *handlers;
 
   GHashTable *request_channel_class_async_ids;
 } EmpathyDispatcherPriv;
 
+static GList *
+empathy_dispatcher_get_channels (EmpathyHandler *handler,
+    gpointer user_data);
+
+static gboolean
+empathy_dispatcher_handle_channels (EmpathyHandler *handler,
+    const gchar *account_path,
+    const gchar *connection_path,
+    const GPtrArray *channels,
+    const GPtrArray *requests_satisfied,
+    guint64 timestamp,
+    GHashTable *handler_info,
+    gpointer user_data,
+    GError **error);
+
 G_DEFINE_TYPE (EmpathyDispatcher, empathy_dispatcher, G_TYPE_OBJECT);
 
+enum
+{
+  PROP_INTERFACES = 1,
+  PROP_HANDLER,
+};
+
 enum
 {
   OBSERVE,
@@ -77,6 +112,10 @@ enum
 static guint signals[LAST_SIGNAL];
 static EmpathyDispatcher *dispatcher = NULL;
 
+static void dispatcher_init_connection_if_needed (
+    EmpathyDispatcher *dispatcher,
+    TpConnection *connection);
+
 static GList * empathy_dispatcher_find_channel_classes
   (EmpathyDispatcher *dispatcher, TpConnection *connection,
    const gchar *channel_type, guint handle_type, GArray *fixed_properties);
@@ -87,10 +126,12 @@ typedef struct
   EmpathyDispatcher *dispatcher;
   EmpathyDispatchOperation *operation;
   TpConnection *connection;
+  gboolean should_ensure;
   gchar *channel_type;
   guint handle_type;
   guint handle;
   EmpathyContact *contact;
+  TpProxyPendingCall *pending_call;
 
   /* Properties to pass to the channel when requesting it */
   GHashTable *request;
@@ -140,6 +181,11 @@ typedef struct
   gpointer user_data;
 } FindChannelRequest;
 
+static void
+empathy_dispatcher_call_create_or_ensure_channel (
+    EmpathyDispatcher *dispatcher,
+    DispatcherRequestData *request_data);
+
 static DispatchData *
 new_dispatch_data (TpChannel *channel,
                    GObject *channel_wrapper)
@@ -178,6 +224,8 @@ new_dispatcher_request_data (EmpathyDispatcher *dispatcher,
   result->dispatcher = g_object_ref (dispatcher);
   result->connection = connection;
 
+  result->should_ensure = FALSE;
+
   result->channel_type = g_strdup (channel_type);
   result->handle_type = handle_type;
   result->handle = handle;
@@ -206,6 +254,9 @@ free_dispatcher_request_data (DispatcherRequestData *r)
   if (r->request != NULL)
     g_hash_table_unref (r->request);
 
+  if (r->pending_call != NULL)
+    tp_proxy_pending_call_cancel (r->pending_call);
+
   g_slice_free (DispatcherRequestData, r);
 }
 
@@ -563,6 +614,8 @@ dispatcher_connection_new_channel (EmpathyDispatcher *dispatcher,
     NULL
   };
 
+  dispatcher_init_connection_if_needed (dispatcher, connection);
+
   cd = g_hash_table_lookup (priv->connections, connection);
 
   /* Don't bother with channels we have already dispatched or are dispatching
@@ -638,30 +691,11 @@ dispatcher_connection_new_channel (EmpathyDispatcher *dispatcher,
 }
 
 static void
-dispatcher_connection_new_channel_cb (TpConnection *connection,
-                                      const gchar *object_path,
-                                      const gchar  *channel_type,
-                                      guint handle_type,
-                                      guint handle,
-                                      gboolean suppress_handler,
-                                      gpointer user_data,
-                                      GObject *object)
-{
-  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (object);
-
-  /* Empathy heavily abuses surpress handler (don't try this at home), if
-   * surpress handler is true then it is an outgoing channel, which is
-   * requested either by us or some other party (like the megaphone applet).
-   * Otherwise it's an incoming channel */
-  dispatcher_connection_new_channel (dispatcher, connection,
-    object_path, channel_type, handle_type, handle, NULL, !suppress_handler);
-}
-
-static void
-dispatcher_connection_new_channel_with_properties (EmpathyDispatcher *dispatcher,
-                                                   TpConnection *connection,
-                                                   const gchar *object_path,
-                                                   GHashTable *properties)
+dispatcher_connection_new_channel_with_properties (
+    EmpathyDispatcher *dispatcher,
+    TpConnection *connection,
+    const gchar *object_path,
+    GHashTable *properties)
 {
   const gchar *channel_type;
   guint handle_type;
@@ -701,36 +735,13 @@ dispatcher_connection_new_channel_with_properties (EmpathyDispatcher *dispatcher
   if (!valid)
     {
       g_message ("%s had an invalid Requested property", object_path);
-      return;
+      requested = FALSE;
     }
 
   dispatcher_connection_new_channel (dispatcher, connection,
     object_path, channel_type, handle_type, handle, properties, !requested);
 }
 
-static void
-dispatcher_connection_new_channels_cb (TpConnection *connection,
-                                       const GPtrArray *channels,
-                                       gpointer user_data,
-                                       GObject *object)
-{
-  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (object);
-  int i;
-
-  for (i = 0; i < channels->len ; i++)
-    {
-      GValueArray *arr = g_ptr_array_index (channels, i);
-      const gchar *object_path;
-      GHashTable *properties;
-
-      object_path = g_value_get_boxed (g_value_array_get_nth (arr, 0));
-      properties = g_value_get_boxed (g_value_array_get_nth (arr, 1));
-
-      dispatcher_connection_new_channel_with_properties (dispatcher,
-        connection, object_path, properties);
-    }
-}
-
 static void
 dispatcher_connection_got_all (TpProxy *proxy,
                                GHashTable *properties,
@@ -740,23 +751,14 @@ dispatcher_connection_got_all (TpProxy *proxy,
 {
   EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (object);
   EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
-  GPtrArray *channels;
   GPtrArray *requestable_channels;
+  GPtrArray *existing_channels;
 
   if (error) {
     DEBUG ("Error: %s", error->message);
     return;
   }
 
-  channels = tp_asv_get_boxed (properties, "Channels",
-    TP_ARRAY_TYPE_CHANNEL_DETAILS_LIST);
-
-  if (channels == NULL)
-    DEBUG ("No Channels property !?! on connection");
-  else
-    dispatcher_connection_new_channels_cb (TP_CONNECTION (proxy),
-      channels, NULL, object);
-
   requestable_channels = tp_asv_get_boxed (properties,
     "RequestableChannelClasses", TP_ARRAY_TYPE_REQUESTABLE_CHANNEL_CLASS_LIST);
 
@@ -795,37 +797,31 @@ dispatcher_connection_got_all (TpProxy *proxy,
 
       g_hash_table_remove (priv->outstanding_classes_requests, proxy);
     }
-}
 
-static void
-dispatcher_connection_list_channels_cb (TpConnection    *connection,
-                                        const GPtrArray *channels,
-                                        const GError    *error,
-                                        gpointer         user_data,
-                                        GObject         *dispatcher)
-{
-  int i;
+  existing_channels = tp_asv_get_boxed (properties,
+      "Channels", TP_ARRAY_TYPE_CHANNEL_DETAILS_LIST);
 
-  if (error)
+  if (existing_channels != NULL)
     {
-      DEBUG ("Error: %s", error->message);
-      return;
-    }
+      int idx;
 
-  for (i = 0; i < channels->len; i++)
-    {
-      GValueArray *values;
+      for (idx = 0; idx < existing_channels->len; idx++)
+        {
+          GValueArray *values = g_ptr_array_index (existing_channels, idx);
+          const gchar *object_path;
+          GHashTable *properties;
 
-      values = g_ptr_array_index (channels, i);
-      /* We don't have any extra info, so assume already existing channels are
-       * incoming... */
-      dispatcher_connection_new_channel (EMPATHY_DISPATCHER (dispatcher),
-        connection,
-        g_value_get_boxed (g_value_array_get_nth (values, 0)),
-        g_value_get_string (g_value_array_get_nth (values, 1)),
-        g_value_get_uint (g_value_array_get_nth (values, 2)),
-        g_value_get_uint (g_value_array_get_nth (values, 3)),
-        NULL, TRUE);
+          object_path = g_value_get_boxed (g_value_array_get_nth (values, 0));
+          properties = g_value_get_boxed (g_value_array_get_nth (values, 1));
+
+          if (tp_strdiff (tp_asv_get_string (properties,
+                      TP_IFACE_CHANNEL ".ChannelType"),
+                  TP_IFACE_CHANNEL_TYPE_TEXT))
+            continue;
+
+          dispatcher_connection_new_channel_with_properties (dispatcher,
+              TP_CONNECTION (proxy), object_path, properties);
+        }
     }
 }
 
@@ -841,9 +837,8 @@ dispatcher_connection_advertise_capabilities_cb (TpConnection    *connection,
 }
 
 static void
-dispatcher_new_connection_cb (EmpathyAccountManager *manager,
-                              TpConnection *connection,
-                              EmpathyDispatcher *dispatcher)
+dispatcher_init_connection_if_needed (EmpathyDispatcher *dispatcher,
+    TpConnection *connection)
 {
   EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
   GPtrArray   *capabilities;
@@ -863,26 +858,11 @@ dispatcher_new_connection_cb (EmpathyAccountManager *manager,
   if (tp_proxy_has_interface_by_id (TP_PROXY (connection),
       TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS))
     {
-      tp_cli_connection_interface_requests_connect_to_new_channels (connection,
-        dispatcher_connection_new_channels_cb,
-        NULL, NULL, G_OBJECT (dispatcher), NULL);
-
       tp_cli_dbus_properties_call_get_all (connection, -1,
         TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
         dispatcher_connection_got_all,
         NULL, NULL, G_OBJECT (dispatcher));
     }
-  else
-    {
-      tp_cli_connection_connect_to_new_channel (connection,
-        dispatcher_connection_new_channel_cb,
-        NULL, NULL, G_OBJECT (dispatcher), NULL);
-
-      tp_cli_connection_call_list_channels (connection, -1,
-        dispatcher_connection_list_channels_cb, NULL, NULL,
-        G_OBJECT (dispatcher));
-
-    }
 
   /* Advertise VoIP capabilities */
   capabilities = g_ptr_array_sized_new (1);
@@ -895,7 +875,9 @@ dispatcher_new_connection_cb (EmpathyAccountManager *manager,
         1, TP_CHANNEL_MEDIA_CAPABILITY_AUDIO |
            TP_CHANNEL_MEDIA_CAPABILITY_VIDEO |
            TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_STUN  |
-           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_GTALK_P2P, G_MAXUINT);
+           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_GTALK_P2P |
+           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_ICE_UDP,
+           G_MAXUINT);
   g_ptr_array_add (capabilities, g_value_get_boxed (&cap));
 
   tp_cli_connection_interface_capabilities_call_advertise_capabilities (
@@ -907,6 +889,14 @@ dispatcher_new_connection_cb (EmpathyAccountManager *manager,
   g_ptr_array_free (capabilities, TRUE);
 }
 
+static void
+dispatcher_new_connection_cb (EmpathyAccountManager *manager,
+                              TpConnection *connection,
+                              EmpathyDispatcher *dispatcher)
+{
+  dispatcher_init_connection_if_needed (dispatcher, connection);
+}
+
 static void
 remove_idle_handlers (gpointer key,
                       gpointer value,
@@ -924,23 +914,69 @@ dispatcher_constructor (GType type,
                         GObjectConstructParam *construct_params)
 {
   GObject *retval;
+  EmpathyDispatcherPriv *priv;
 
-  if (dispatcher == NULL)
-    {
-      retval = G_OBJECT_CLASS (empathy_dispatcher_parent_class)->constructor
-          (type, n_construct_params, construct_params);
+  if (dispatcher != NULL)
+    return g_object_ref (dispatcher);
 
-      dispatcher = EMPATHY_DISPATCHER (retval);
-      g_object_add_weak_pointer (retval, (gpointer) &dispatcher);
-    }
-  else
-    {
-      retval = g_object_ref (dispatcher);
-    }
+  retval = G_OBJECT_CLASS (empathy_dispatcher_parent_class)->constructor
+    (type, n_construct_params, construct_params);
+
+  dispatcher = EMPATHY_DISPATCHER (retval);
+  g_object_add_weak_pointer (retval, (gpointer) &dispatcher);
+
+  priv = GET_PRIV (dispatcher);
+
+  if (priv->handler == NULL)
+    priv->handler = empathy_handler_new (NULL, NULL, NULL);
+
+  empathy_handler_set_handle_channels_func (priv->handler,
+    empathy_dispatcher_handle_channels,
+    dispatcher);
+
+  empathy_handler_set_channels_func (priv->handler,
+    empathy_dispatcher_get_channels,
+    dispatcher);
 
   return retval;
 }
 
+static void
+dispatcher_dispose (GObject *object)
+{
+  EmpathyDispatcherPriv *priv = GET_PRIV (object);
+  GHashTableIter iter;
+  gpointer connection;
+  GList *l;
+
+  if (priv->dispose_has_run)
+    return;
+
+  priv->dispose_has_run = TRUE;
+
+  for (l = priv->handlers ; l != NULL; l = g_list_next (l))
+    g_object_unref (G_OBJECT (l->data));
+
+  g_list_free (priv->handlers);
+  priv->handlers = NULL;
+
+  if (priv->handler != NULL)
+    g_object_unref (priv->handler);
+  priv->handler = NULL;
+
+  g_hash_table_iter_init (&iter, priv->connections);
+  while (g_hash_table_iter_next (&iter, &connection, NULL))
+    {
+      g_signal_handlers_disconnect_by_func (connection,
+          dispatcher_connection_invalidated_cb, object);
+    }
+
+  g_hash_table_destroy (priv->connections);
+  priv->connections = NULL;
+
+  G_OBJECT_CLASS (empathy_dispatcher_parent_class)->dispose (object);
+}
+
 static void
 dispatcher_finalize (GObject *object)
 {
@@ -968,13 +1004,6 @@ dispatcher_finalize (GObject *object)
 
   g_list_free (priv->channels);
 
-  g_hash_table_iter_init (&iter, priv->connections);
-  while (g_hash_table_iter_next (&iter, &connection, NULL))
-    {
-      g_signal_handlers_disconnect_by_func (connection,
-          dispatcher_connection_invalidated_cb, object);
-    }
-
   g_hash_table_iter_init (&iter, priv->outstanding_classes_requests);
   while (g_hash_table_iter_next (&iter, &connection, (gpointer *) &list))
     {
@@ -983,20 +1012,70 @@ dispatcher_finalize (GObject *object)
     }
 
   g_object_unref (priv->account_manager);
-  g_object_unref (priv->mc);
 
-  g_hash_table_destroy (priv->connections);
   g_hash_table_destroy (priv->outstanding_classes_requests);
 }
 
+static void
+dispatcher_set_property (GObject *object,
+  guint property_id,
+  const GValue *value,
+  GParamSpec *pspec)
+{
+  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (object);
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+
+  switch (property_id)
+    {
+      case PROP_HANDLER:
+        priv->handler = g_value_dup_object (value);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
+}
+
+static void
+dispatcher_get_property (GObject *object,
+  guint property_id,
+  GValue *value,
+  GParamSpec *pspec)
+{
+  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (object);
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+
+  switch (property_id)
+    {
+      case PROP_HANDLER:
+        g_value_set_object (value, priv->handler);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
+}
+
 static void
 empathy_dispatcher_class_init (EmpathyDispatcherClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GParamSpec *param_spec;
 
+  object_class->dispose = dispatcher_dispose;
   object_class->finalize = dispatcher_finalize;
   object_class->constructor = dispatcher_constructor;
 
+  object_class->get_property = dispatcher_get_property;
+  object_class->set_property = dispatcher_set_property;
+
+  param_spec = g_param_spec_object ("handler", "handler",
+    "The main Telepathy Client Hander object",
+    EMPATHY_TYPE_HANDLER,
+    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT_ONLY);
+  g_object_class_install_property (object_class,
+    PROP_HANDLER, param_spec);
+
   signals[OBSERVE] =
     g_signal_new ("observe",
       G_TYPE_FROM_CLASS (klass),
@@ -1027,8 +1106,8 @@ empathy_dispatcher_class_init (EmpathyDispatcherClass *klass)
       G_TYPE_NONE,
       1, EMPATHY_TYPE_DISPATCH_OPERATION);
 
-  g_type_class_add_private (object_class, sizeof (EmpathyDispatcherPriv));
 
+  g_type_class_add_private (object_class, sizeof (EmpathyDispatcherPriv));
 }
 
 static void
@@ -1039,7 +1118,6 @@ empathy_dispatcher_init (EmpathyDispatcher *dispatcher)
     EMPATHY_TYPE_DISPATCHER, EmpathyDispatcherPriv);
 
   dispatcher->priv = priv;
-  priv->mc = empathy_mission_control_dup_singleton ();
   priv->account_manager = empathy_account_manager_dup_singleton ();
 
   g_signal_connect (priv->account_manager, "new-connection",
@@ -1054,10 +1132,12 @@ empathy_dispatcher_init (EmpathyDispatcher *dispatcher)
 
   priv->channels = NULL;
 
-  connections = empathy_account_manager_dup_connections (priv->account_manager);
+  connections = empathy_account_manager_dup_connections (
+      priv->account_manager);
   for (l = connections; l; l = l->next)
     {
-      dispatcher_new_connection_cb (priv->account_manager, l->data, dispatcher);
+      dispatcher_new_connection_cb (priv->account_manager, l->data,
+          dispatcher);
       g_object_unref (l->data);
     }
   g_list_free (connections);
@@ -1066,6 +1146,26 @@ empathy_dispatcher_init (EmpathyDispatcher *dispatcher)
     g_direct_equal);
 }
 
+EmpathyDispatcher *
+empathy_dispatcher_new (const gchar *name,
+  GPtrArray *filters,
+  GStrv capabilities)
+{
+  g_assert (dispatcher == NULL);
+  EmpathyHandler *handler;
+  EmpathyDispatcher *ret;
+
+  handler = empathy_handler_new (name, filters, capabilities);
+
+  ret = EMPATHY_DISPATCHER (
+    g_object_new (EMPATHY_TYPE_DISPATCHER,
+      "handler", handler,
+      NULL));
+  g_object_unref (handler);
+
+  return ret;
+}
+
 EmpathyDispatcher *
 empathy_dispatcher_dup_singleton (void)
 {
@@ -1080,21 +1180,27 @@ dispatcher_request_failed (EmpathyDispatcher *dispatcher,
   EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
   ConnectionData *conn_data;
 
-  conn_data = g_hash_table_lookup (priv->connections, request_data->connection);
+  conn_data = g_hash_table_lookup (priv->connections,
+      request_data->connection);
   if (request_data->cb != NULL)
     request_data->cb (NULL, error, request_data->user_data);
 
-  conn_data->outstanding_requests =
-      g_list_remove (conn_data->outstanding_requests, request_data);
+  if (conn_data != NULL)
+    {
+      conn_data->outstanding_requests =
+          g_list_remove (conn_data->outstanding_requests, request_data);
+    }
+  /* else Connection has been invalidated */
+
   free_dispatcher_request_data (request_data);
 }
 
 static void
 dispatcher_connection_new_requested_channel (EmpathyDispatcher *dispatcher,
-                                             DispatcherRequestData *request_data,
-                                             const gchar *object_path,
-                                             GHashTable *properties,
-                                             const GError *error)
+  DispatcherRequestData *request_data,
+  const gchar *object_path,
+  GHashTable *properties,
+  const GError *error)
 {
   EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
   EmpathyDispatchOperation *operation = NULL;
@@ -1193,8 +1299,11 @@ dispatcher_request_channel_cb (TpConnection *connection,
                                gpointer user_data,
                                GObject *weak_object)
 {
-  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (weak_object);
   DispatcherRequestData *request_data = (DispatcherRequestData *) user_data;
+  EmpathyDispatcher *dispatcher =
+      EMPATHY_DISPATCHER (request_data->dispatcher);
+
+  request_data->pending_call = NULL;
 
   dispatcher_connection_new_requested_channel (dispatcher,
     request_data, object_path, NULL, error);
@@ -1203,12 +1312,35 @@ dispatcher_request_channel_cb (TpConnection *connection,
 static void
 dispatcher_request_channel (DispatcherRequestData *request_data)
 {
-  tp_cli_connection_call_request_channel (request_data->connection, -1,
-    request_data->channel_type,
-    request_data->handle_type,
-    request_data->handle,
-    TRUE, dispatcher_request_channel_cb,
-    request_data, NULL, G_OBJECT (request_data->dispatcher));
+  if (tp_proxy_has_interface_by_id (TP_PROXY (request_data->connection),
+      TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS))
+    {
+      /* Extend the request_data to be a valid request */
+      g_assert (request_data->request == NULL);
+      request_data->request = tp_asv_new (
+        TP_IFACE_CHANNEL ".ChannelType",
+          G_TYPE_STRING, request_data->channel_type,
+        TP_IFACE_CHANNEL ".TargetHandleType",
+          G_TYPE_UINT, request_data->handle_type,
+        NULL);
+
+      if (request_data->handle_type != TP_HANDLE_TYPE_NONE)
+        tp_asv_set_uint32 (request_data->request,
+          TP_IFACE_CHANNEL ".TargetHandle", request_data->handle);
+
+      empathy_dispatcher_call_create_or_ensure_channel (
+        request_data->dispatcher, request_data);
+    }
+  else
+    {
+      request_data->pending_call = tp_cli_connection_call_request_channel (
+        request_data->connection, -1,
+        request_data->channel_type,
+        request_data->handle_type,
+        request_data->handle,
+        TRUE, dispatcher_request_channel_cb,
+        request_data, NULL, NULL);
+    }
 }
 
 void
@@ -1231,9 +1363,10 @@ empathy_dispatcher_chat_with_contact (EmpathyContact *contact,
   connection_data = g_hash_table_lookup (priv->connections, connection);
 
   /* The contact handle might not be known yet */
-  request_data  = new_dispatcher_request_data (dispatcher, connection,
+  request_data = new_dispatcher_request_data (dispatcher, connection,
     TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_CONTACT,
     empathy_contact_get_handle (contact), NULL, contact, callback, user_data);
+  request_data->should_ensure = TRUE;
 
   connection_data->outstanding_requests = g_list_prepend
     (connection_data->outstanding_requests, request_data);
@@ -1266,7 +1399,8 @@ dispatcher_chat_with_contact_id_cb (EmpathyTpContactFactory *factory,
     }
   else
     {
-      empathy_dispatcher_chat_with_contact (contact, data->callback, data->user_data);
+      empathy_dispatcher_chat_with_contact (contact, data->callback,
+          data->user_data);
     }
 
   g_object_unref (data->dispatcher);
@@ -1307,6 +1441,8 @@ dispatcher_request_handles_cb (TpConnection *connection,
 {
   DispatcherRequestData *request_data = (DispatcherRequestData *) user_data;
 
+  request_data->pending_call = NULL;
+
   if (error != NULL)
     {
       EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (object);
@@ -1359,10 +1495,10 @@ empathy_dispatcher_join_muc (TpConnection *connection,
   connection_data->outstanding_requests = g_list_prepend
     (connection_data->outstanding_requests, request_data);
 
-  tp_cli_connection_call_request_handles (connection, -1,
+  request_data->pending_call = tp_cli_connection_call_request_handles (
+    connection, -1,
     TP_HANDLE_TYPE_ROOM, names,
-    dispatcher_request_handles_cb, request_data, NULL,
-    G_OBJECT (dispatcher));
+    dispatcher_request_handles_cb, request_data, NULL, NULL);
 
   g_object_unref (dispatcher);
 }
@@ -1375,13 +1511,58 @@ dispatcher_create_channel_cb (TpConnection *connect,
                               gpointer user_data,
                               GObject *weak_object)
 {
-  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (weak_object);
   DispatcherRequestData *request_data = (DispatcherRequestData *) user_data;
+  EmpathyDispatcher *dispatcher =
+      EMPATHY_DISPATCHER (request_data->dispatcher);
+
+  request_data->pending_call = NULL;
 
   dispatcher_connection_new_requested_channel (dispatcher,
     request_data, object_path, properties, error);
 }
 
+static void
+dispatcher_ensure_channel_cb (TpConnection *connect,
+                              gboolean is_ours,
+                              const gchar *object_path,
+                              GHashTable *properties,
+                              const GError *error,
+                              gpointer user_data,
+                              GObject *weak_object)
+{
+  DispatcherRequestData *request_data = (DispatcherRequestData *) user_data;
+  EmpathyDispatcher *dispatcher =
+      EMPATHY_DISPATCHER (request_data->dispatcher);
+
+  request_data->pending_call = NULL;
+
+  dispatcher_connection_new_requested_channel (dispatcher,
+    request_data, object_path, properties, error);
+}
+
+static void
+empathy_dispatcher_call_create_or_ensure_channel (
+    EmpathyDispatcher *dispatcher,
+    DispatcherRequestData *request_data)
+{
+  if (request_data->should_ensure)
+    {
+      request_data->pending_call =
+          tp_cli_connection_interface_requests_call_ensure_channel (
+          request_data->connection, -1,
+          request_data->request, dispatcher_ensure_channel_cb,
+          request_data, NULL, NULL);
+    }
+  else
+    {
+      request_data->pending_call =
+          tp_cli_connection_interface_requests_call_create_channel (
+          request_data->connection, -1,
+          request_data->request, dispatcher_create_channel_cb,
+          request_data, NULL, NULL);
+    }
+}
+
 void
 empathy_dispatcher_create_channel (EmpathyDispatcher *dispatcher,
                                    TpConnection *connection,
@@ -1420,10 +1601,7 @@ empathy_dispatcher_create_channel (EmpathyDispatcher *dispatcher,
   connection_data->outstanding_requests = g_list_prepend
     (connection_data->outstanding_requests, request_data);
 
-  tp_cli_connection_interface_requests_call_create_channel (
-    request_data->connection, -1,
-    request_data->request, dispatcher_create_channel_cb, request_data, NULL,
-    G_OBJECT (request_data->dispatcher));
+  empathy_dispatcher_call_create_or_ensure_channel (dispatcher, request_data);
 }
 
 static gboolean
@@ -1740,7 +1918,7 @@ empathy_dispatcher_find_requestable_channel_classes_async
 
   /* append another request for this connection */
   request = g_slice_new0 (FindChannelRequest);
-  request->dispatcher = dispatcher;
+  request->dispatcher = g_object_ref (dispatcher);
   request->channel_type = g_strdup (channel_type);
   request->handle_type = handle_type;
   request->connection = connection;
@@ -1753,3 +1931,97 @@ empathy_dispatcher_find_requestable_channel_classes_async
   g_hash_table_insert (priv->request_channel_class_async_ids,
     request, GUINT_TO_POINTER (source_id));
 }
+
+static GList *
+empathy_dispatcher_get_channels (EmpathyHandler *handler,
+  gpointer user_data)
+{
+  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (user_data);
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+
+  return priv->channels;
+}
+
+static gboolean
+empathy_dispatcher_handle_channels (EmpathyHandler *handler,
+    const gchar *account_path,
+    const gchar *connection_path,
+    const GPtrArray *channels,
+    const GPtrArray *requests_satisfied,
+    guint64 timestamp,
+    GHashTable *handler_info,
+    gpointer user_data,
+    GError **error)
+{
+  EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (user_data);
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+  int i;
+  EmpathyAccount *account;
+  TpConnection *connection;
+
+  account = empathy_account_manager_ensure_account (priv->account_manager,
+    account_path);
+  g_assert (account != NULL);
+
+  connection = empathy_account_get_connection_for_path (account,
+      connection_path);
+  if (connection == NULL)
+    {
+      g_set_error_literal (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+        "Invalid connection argument");
+      return FALSE;
+    }
+
+  for (i = 0; i < channels->len ; i++)
+    {
+      GValueArray *arr = g_ptr_array_index (channels, i);
+      const gchar *object_path;
+      GHashTable *properties;
+
+      object_path = g_value_get_boxed (g_value_array_get_nth (arr, 0));
+      properties = g_value_get_boxed (g_value_array_get_nth (arr, 1));
+
+      dispatcher_connection_new_channel_with_properties (dispatcher,
+        connection, object_path, properties);
+    }
+
+  return TRUE;
+}
+
+
+EmpathyHandler *
+empathy_dispatcher_add_handler (EmpathyDispatcher *dispatcher,
+    const gchar *name,
+    GPtrArray *filters,
+    GStrv capabilities)
+{
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+  EmpathyHandler *handler;
+
+  handler = empathy_handler_new (name, filters, capabilities);
+  priv->handlers = g_list_prepend (priv->handlers, handler);
+
+  /* Only set the handle_channels function, the Channel property on the main
+   * handler will always report all dispatched channels even if they came from
+   * a different Handler */
+  empathy_handler_set_handle_channels_func (handler,
+    empathy_dispatcher_handle_channels,
+    dispatcher);
+
+  return handler;
+}
+
+void
+empathy_dispatcher_remove_handler (EmpathyDispatcher *dispatcher,
+  EmpathyHandler *handler)
+{
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
+  GList *h;
+
+  h = g_list_find (priv->handlers, handler);
+  g_return_if_fail (h != NULL);
+
+  priv->handlers = g_list_delete_link (priv->handlers, h);
+
+  g_object_unref (handler);
+}
