@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Richard Hult <richard@imendio.com>
  *          Martyn Russell <martyn@imendio.com>
@@ -35,9 +35,9 @@
 #include <telepathy-glib/connection.h>
 #include <telepathy-glib/channel.h>
 #include <telepathy-glib/dbus.h>
+#include <telepathy-glib/util.h>
 
 #include "empathy-utils.h"
-#include "empathy-contact-factory.h"
 #include "empathy-contact-manager.h"
 #include "empathy-dispatcher.h"
 #include "empathy-dispatch-operation.h"
@@ -49,6 +49,28 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include "empathy-debug.h"
 
+/* Translation between presence types and string */
+static struct {
+	gchar *name;
+	TpConnectionPresenceType type;
+} presence_types[] = {
+	{ "available", TP_CONNECTION_PRESENCE_TYPE_AVAILABLE },
+	{ "busy",      TP_CONNECTION_PRESENCE_TYPE_BUSY },
+	{ "away",      TP_CONNECTION_PRESENCE_TYPE_AWAY },
+	{ "ext_away",  TP_CONNECTION_PRESENCE_TYPE_EXTENDED_AWAY },
+	{ "hidden",    TP_CONNECTION_PRESENCE_TYPE_HIDDEN },
+	{ "offline",   TP_CONNECTION_PRESENCE_TYPE_OFFLINE },
+	{ "unset",     TP_CONNECTION_PRESENCE_TYPE_UNSET },
+	{ "unknown",   TP_CONNECTION_PRESENCE_TYPE_UNKNOWN },
+	{ "error",     TP_CONNECTION_PRESENCE_TYPE_ERROR },
+	/* alternative names */
+	{ "dnd",      TP_CONNECTION_PRESENCE_TYPE_BUSY },
+	{ "brb",      TP_CONNECTION_PRESENCE_TYPE_AWAY },
+	{ "xa",       TP_CONNECTION_PRESENCE_TYPE_EXTENDED_AWAY },
+	{ NULL, },
+};
+
+
 
 void
 empathy_init (void)
@@ -141,7 +163,7 @@ empathy_xml_validate (xmlDoc      *doc,
 }
 
 xmlNodePtr
-empathy_xml_node_get_child (xmlNodePtr   node, 
+empathy_xml_node_get_child (xmlNodePtr   node,
 			   const gchar *child_name)
 {
 	xmlNodePtr l;
@@ -159,7 +181,7 @@ empathy_xml_node_get_child (xmlNodePtr   node,
 }
 
 xmlChar *
-empathy_xml_node_get_child_content (xmlNodePtr   node, 
+empathy_xml_node_get_child_content (xmlNodePtr   node,
 				   const gchar *child_name)
 {
 	xmlNodePtr l;
@@ -171,12 +193,12 @@ empathy_xml_node_get_child_content (xmlNodePtr   node,
 	if (l) {
 		return xmlNodeGetContent (l);
 	}
-		
+
 	return NULL;
 }
 
 xmlNodePtr
-empathy_xml_node_find_child_prop_value (xmlNodePtr   node, 
+empathy_xml_node_find_child_prop_value (xmlNodePtr   node,
 				       const gchar *prop_name,
 				       const gchar *prop_value)
 {
@@ -198,120 +220,59 @@ empathy_xml_node_find_child_prop_value (xmlNodePtr   node,
 		if (prop && strcmp (prop, prop_value) == 0) {
 			found = l;
 		}
-		
+
 		xmlFree (prop);
 	}
-		
+
 	return found;
 }
 
-guint
-empathy_account_hash (gconstpointer key)
-{
-	g_return_val_if_fail (MC_IS_ACCOUNT (key), 0);
-
-	return g_str_hash (mc_account_get_unique_name (MC_ACCOUNT (key)));
-}
-
-gboolean
-empathy_account_equal (gconstpointer a,
-		       gconstpointer b)
-{
-	const gchar *name_a;
-	const gchar *name_b;
-
-	g_return_val_if_fail (MC_IS_ACCOUNT (a), FALSE);
-	g_return_val_if_fail (MC_IS_ACCOUNT (b), FALSE);
-
-	name_a = mc_account_get_unique_name (MC_ACCOUNT (a));
-	name_b = mc_account_get_unique_name (MC_ACCOUNT (b));
-
-	return g_str_equal (name_a, name_b);
-}
-
-MissionControl *
-empathy_mission_control_dup_singleton (void)
-{
-	static MissionControl *mc = NULL;
-
-	if (!mc) {
-		mc = mission_control_new (tp_get_bus ());
-		g_object_add_weak_pointer (G_OBJECT (mc), (gpointer) &mc);
-	} else {
-		g_object_ref (mc);
-	}
-
-	return mc;
-}
-
 const gchar *
-empathy_presence_get_default_message (McPresence presence)
+empathy_presence_get_default_message (TpConnectionPresenceType presence)
 {
 	switch (presence) {
-	case MC_PRESENCE_AVAILABLE:
+	case TP_CONNECTION_PRESENCE_TYPE_AVAILABLE:
 		return _("Available");
-	case MC_PRESENCE_DO_NOT_DISTURB:
+	case TP_CONNECTION_PRESENCE_TYPE_BUSY:
 		return _("Busy");
-	case MC_PRESENCE_AWAY:
-	case MC_PRESENCE_EXTENDED_AWAY:
+	case TP_CONNECTION_PRESENCE_TYPE_AWAY:
+	case TP_CONNECTION_PRESENCE_TYPE_EXTENDED_AWAY:
 		return _("Away");
-	case MC_PRESENCE_HIDDEN:
+	case TP_CONNECTION_PRESENCE_TYPE_HIDDEN:
 		return _("Hidden");
-	case MC_PRESENCE_OFFLINE:
-	case MC_PRESENCE_UNSET:
+	case TP_CONNECTION_PRESENCE_TYPE_OFFLINE:
 		return _("Offline");
-	default:
-		g_assert_not_reached ();
+	case TP_CONNECTION_PRESENCE_TYPE_UNSET:
+	case TP_CONNECTION_PRESENCE_TYPE_UNKNOWN:
+	case TP_CONNECTION_PRESENCE_TYPE_ERROR:
+		return NULL;
 	}
 
 	return NULL;
 }
 
 const gchar *
-empathy_presence_to_str (McPresence presence)
+empathy_presence_to_str (TpConnectionPresenceType presence)
 {
-	switch (presence) {
-	case MC_PRESENCE_AVAILABLE:
-		return "available";
-	case MC_PRESENCE_DO_NOT_DISTURB:
-		return "busy";
-	case MC_PRESENCE_AWAY:
-		return "away";
-	case MC_PRESENCE_EXTENDED_AWAY:
-		return "ext_away";
-	case MC_PRESENCE_HIDDEN:
-		return "hidden";
-	case MC_PRESENCE_OFFLINE:
-		return "offline";
-	case MC_PRESENCE_UNSET:
-		return "unset";
-	default:
-		g_assert_not_reached ();
-	}
+	int i;
+
+	for (i = 0 ; presence_types[i].name != NULL; i++)
+		if (presence == presence_types[i].type)
+			return presence_types[i].name;
 
 	return NULL;
 }
 
-McPresence
+TpConnectionPresenceType
 empathy_presence_from_str (const gchar *str)
 {
-	if (strcmp (str, "available") == 0) {
-		return MC_PRESENCE_AVAILABLE;
-	} else if ((strcmp (str, "dnd") == 0) || (strcmp (str, "busy") == 0)) {
-		return MC_PRESENCE_DO_NOT_DISTURB;
-	} else if ((strcmp (str, "away") == 0) || (strcmp (str, "brb") == 0)) {
-		return MC_PRESENCE_AWAY;
-	} else if ((strcmp (str, "xa") == 0) || (strcmp (str, "ext_away") == 0)) {
-		return MC_PRESENCE_EXTENDED_AWAY;
-	} else if (strcmp (str, "hidden") == 0) {
-		return MC_PRESENCE_HIDDEN;
-	} else if (strcmp (str, "offline") == 0) {
-		return MC_PRESENCE_OFFLINE;
-	} else if (strcmp (str, "unset") == 0) {
-		return MC_PRESENCE_UNSET;
-	}
+	int i;
 
-	return MC_PRESENCE_UNSET;
+	for (i = 0 ; presence_types[i].name != NULL; i++)
+		if (!tp_strdiff (str, presence_types[i].name))
+			return presence_types[i].type;
+
+	return TP_CONNECTION_PRESENCE_TYPE_UNSET;
 }
 
 gchar *
@@ -332,97 +293,6 @@ empathy_file_lookup (const gchar *filename, const gchar *subdir)
 	return path;
 }
 
-typedef struct {
-	EmpathyRunUntilReadyFunc  func;
-	gpointer                  user_data;
-	GObject                  *object;
-	GMainLoop                *loop;
-} RunUntilReadyData;
-
-static void
-run_until_ready_cb (RunUntilReadyData *data)
-{
-	if (!data->func || data->func (data->object, data->user_data)) {
-		DEBUG ("Object %p is ready", data->object);
-		g_main_loop_quit (data->loop);
-	}
-}
-
-static gboolean
-object_is_ready (GObject *object,
-		 gpointer user_data)
-{
-	gboolean ready;
-
-	g_object_get (object, "ready", &ready, NULL);
-
-	return ready;
-}
-
-void
-empathy_run_until_ready_full (gpointer                  object,
-			      const gchar              *signal,
-			      EmpathyRunUntilReadyFunc  func,
-			      gpointer                  user_data,
-			      GMainLoop               **loop)
-{
-	RunUntilReadyData  data;
-	gulong             signal_id;
-
-	g_return_if_fail (G_IS_OBJECT (object));
-	g_return_if_fail (signal != NULL);
-
-	if (func && func (object, user_data)) {
-		return;
-	}
-
-	DEBUG ("Starting run until ready for object %p", object);
-
-	data.func = func;
-	data.user_data = user_data;
-	data.object = object;
-	data.loop = g_main_loop_new (NULL, FALSE);
-
-	signal_id = g_signal_connect_swapped (object, signal,
-					      G_CALLBACK (run_until_ready_cb),
-					      &data);
-	if (loop != NULL) {
-		*loop = data.loop;
-	}
-
-	g_main_loop_run (data.loop);
-
-	if (loop != NULL) {
-		*loop = NULL;
-	}
-
-	g_signal_handler_disconnect (object, signal_id);
-	g_main_loop_unref (data.loop);
-}
-
-void
-empathy_run_until_ready (gpointer object)
-{
-	empathy_run_until_ready_full (object, "notify::ready", object_is_ready,
-				      NULL, NULL);
-}
-
-McAccount *
-empathy_channel_get_account (TpChannel *channel)
-{
-	TpConnection   *connection;
-	McAccount      *account;
-	MissionControl *mc;
-
-	g_object_get (channel, "connection", &connection, NULL);
-	mc = empathy_mission_control_dup_singleton ();
-	account = mission_control_get_account_for_tpconnection (mc, connection, NULL);
-	g_object_unref (connection);
-	g_object_unref (mc);
-
-	return account;
-}
-
 guint
 empathy_proxy_hash (gconstpointer key)
 {
@@ -456,17 +326,165 @@ empathy_proxy_equal (gconstpointer a,
 gboolean
 empathy_check_available_state (void)
 {
-	McPresence presence;
+	TpConnectionPresenceType presence;
 	EmpathyIdle *idle;
 
 	idle = empathy_idle_dup_singleton ();
 	presence = empathy_idle_get_state (idle);
 	g_object_unref (idle);
 
-	if (presence != MC_PRESENCE_AVAILABLE &&
-		presence != MC_PRESENCE_UNSET) {
-		return FALSE;    
+	if (presence != TP_CONNECTION_PRESENCE_TYPE_AVAILABLE &&
+		presence != TP_CONNECTION_PRESENCE_TYPE_UNSET) {
+		return FALSE;
 	}
 
 	return TRUE;
 }
+
+gint
+empathy_uint_compare (gconstpointer a,
+		      gconstpointer b)
+{
+	return *(guint *) a - *(guint *) b;
+}
+
+gchar *
+empathy_protocol_icon_name (const gchar *protocol)
+{
+  return g_strdup_printf ("im-%s", protocol);
+}
+
+GType
+empathy_type_dbus_ao (void)
+{
+  static GType t = 0;
+
+  if (G_UNLIKELY (t == 0))
+     t = dbus_g_type_get_collection ("GPtrArray", DBUS_TYPE_G_OBJECT_PATH);
+
+  return t;
+}
+
+const char *
+empathy_protocol_name_to_display_name (const gchar *proto_name)
+{
+  int i;
+  static struct {
+    const gchar *proto;
+    const gchar *display;
+    gboolean translated;
+  } names[] = {
+    { "jabber", "Jabber", FALSE },
+    { "gtalk", "Google Talk", FALSE },
+    { "msn", "MSN", FALSE, },
+    { "local-xmpp", N_("People Nearby"), TRUE },
+    { "irc", "IRC", FALSE },
+    { "icq", "ICQ", FALSE },
+    { "aim", "AIM", FALSE },
+    { "yahoo", "Yahoo!", FALSE },
+    { "yahoojp", N_("Yahoo! Japan"), TRUE },
+    { "facebook", N_("Facebook Chat"), TRUE },
+    { "groupwise", "GroupWise", FALSE },
+    { "sip", "SIP", FALSE },
+    { NULL, NULL }
+  };
+
+  for (i = 0; names[i].proto != NULL; i++)
+    {
+      if (!tp_strdiff (proto_name, names[i].proto))
+        {
+          if (names[i].translated)
+            return _(names[i].display);
+          else
+            return names[i].display;
+        }
+    }
+
+  return NULL;
+}
+
+typedef struct {
+    GObject *instance;
+    GObject *user_data;
+    gulong handler_id;
+} WeakHandlerCtx;
+
+static WeakHandlerCtx *
+whc_new (GObject *instance,
+         GObject *user_data)
+{
+  WeakHandlerCtx *ctx = g_slice_new0 (WeakHandlerCtx);
+
+  ctx->instance = instance;
+  ctx->user_data = user_data;
+
+  return ctx;
+}
+
+static void
+whc_free (WeakHandlerCtx *ctx)
+{
+  g_slice_free (WeakHandlerCtx, ctx);
+}
+
+static void user_data_destroyed_cb (gpointer, GObject *);
+
+static void
+instance_destroyed_cb (gpointer ctx_,
+                       GObject *where_the_instance_was)
+{
+  WeakHandlerCtx *ctx = ctx_;
+
+  DEBUG ("instance for %p destroyed; cleaning up", ctx);
+
+  /* No need to disconnect the signal here, the instance has gone away. */
+  g_object_weak_unref (ctx->user_data, user_data_destroyed_cb, ctx);
+  whc_free (ctx);
+}
+
+static void
+user_data_destroyed_cb (gpointer ctx_,
+                        GObject *where_the_user_data_was)
+{
+  WeakHandlerCtx *ctx = ctx_;
+
+  DEBUG ("user_data for %p destroyed; disconnecting", ctx);
+
+  g_signal_handler_disconnect (ctx->instance, ctx->handler_id);
+  g_object_weak_unref (ctx->instance, instance_destroyed_cb, ctx);
+  whc_free (ctx);
+}
+
+/* This function is copied from telepathy-gabble: util.c */
+/**
+ * empathy_signal_connect_weak:
+ * @instance: the instance to connect to.
+ * @detailed_signal: a string of the form "signal-name::detail".
+ * @c_handler: the GCallback to connect.
+ * @user_data: an object to pass as data to c_handler calls.
+ *
+ * Connects a #GCallback function to a signal for a particular object, as if
+ * with g_signal_connect(). Additionally, arranges for the signal handler to be
+ * disconnected if @user_data is destroyed.
+ *
+ * This is intended to be a convenient way for objects to use themselves as
+ * user_data for callbacks without having to explicitly disconnect all the
+ * handlers in their finalizers.
+ */
+void
+empathy_signal_connect_weak (gpointer instance,
+    const gchar *detailed_signal,
+    GCallback c_handler,
+    GObject *user_data)
+{
+  GObject *instance_obj = G_OBJECT (instance);
+  WeakHandlerCtx *ctx = whc_new (instance_obj, user_data);
+
+  DEBUG ("connecting to %p:%s with context %p", instance, detailed_signal, ctx);
+
+  ctx->handler_id = g_signal_connect (instance, detailed_signal, c_handler,
+      user_data);
+
+  g_object_weak_ref (instance_obj, instance_destroyed_cb, ctx);
+  g_object_weak_ref (user_data, user_data_destroyed_cb, ctx);
+}
