@@ -135,11 +135,6 @@ enum {
   PROP_PARENT = 1
 };
 
-static void accounts_dialog_account_display_name_changed_cb (
-    EmpathyAccount *account,
-    GParamSpec *pspec,
-    gpointer user_data);
-
 static EmpathyAccountSettings * accounts_dialog_model_get_selected_settings (
     EmpathyAccountsDialog *dialog);
 
@@ -215,12 +210,15 @@ static gchar *
 get_default_display_name (EmpathyAccountSettings *settings)
 {
   const gchar *login_id;
-  const gchar *protocol;
+  const gchar *protocol, *p;
   gchar *default_display_name;
 
   login_id = empathy_account_settings_get_string (settings, "account");
   protocol = empathy_account_settings_get_protocol (settings);
 
+  if ((p = empathy_protocol_name_to_display_name (protocol)) != NULL)
+    protocol = p;
+
   if (login_id != NULL)
     {
       if (!tp_strdiff (protocol, "irc"))
@@ -393,7 +391,7 @@ accounts_dialog_protocol_changed_cb (GtkWidget *widget,
       return;
     }
 
-#ifndef HAVE_NBTK
+#ifndef HAVE_MOBLIN
   if (tp_connection_manager_protocol_can_register (proto) && !is_gtalk)
     {
       gtk_widget_show (priv->radiobutton_register);
@@ -758,8 +756,6 @@ accounts_dialog_delete_account_response_cb (GtkDialog *message_dialog,
 
       if (account != NULL)
         {
-          g_signal_handlers_disconnect_by_func (account,
-              accounts_dialog_account_display_name_changed_cb, account_dialog);
           empathy_account_remove_async (account, NULL, NULL);
           g_object_unref (account);
           account = NULL;
@@ -798,17 +794,17 @@ accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
     }
 
   question_dialog_primary_text = g_strdup_printf (
-#ifndef HAVE_NBTK
+#ifndef HAVE_MOBLIN
       _("You are about to remove your %s account!\n"
           "Are you sure you want to proceed?"),
 #else
       /* Translators: this is used only when built on a moblin platform */
       _("Do you want to remove %s from your computer?"),
-#endif /* HAVE_NBTK */
+#endif /* HAVE_MOBLIN */
       empathy_account_get_display_name (account));
 
   accounts_dialog_show_question_dialog (dialog, question_dialog_primary_text,
-#ifndef HAVE_NBTK
+#ifndef HAVE_MOBLIN
       _("Any associated conversations and chat rooms will NOT be "
           "removed if you decide to proceed.\n"
           "\n"
@@ -817,7 +813,7 @@ accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
 #else
       /* Translators: this is used only when built on a moblin platform */
       _("This will not remove your account on the server."),
-#endif /* HAVE_NBTK */
+#endif /* HAVE_MOBLIN */
       G_CALLBACK (accounts_dialog_delete_account_response_cb),
       dialog,
       GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
@@ -856,7 +852,7 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
       accounts_dialog_model_pixbuf_data_func,
       dialog,
       NULL);
-#ifdef HAVE_NBTK
+#ifdef HAVE_MOBLIN
   g_object_set (cell, "ypad", 4, NULL);
 #endif
 
@@ -881,7 +877,7 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
   gtk_tree_view_column_pack_start (column, cell, FALSE);
   g_object_set (cell,
         "icon-name", GTK_STOCK_DELETE,
-#ifdef HAVE_NBTK
+#ifdef HAVE_MOBLIN
         "show-on-select", TRUE,
 #endif
         NULL);
@@ -1223,6 +1219,9 @@ accounts_dialog_account_display_name_changed_cb (EmpathyAccount *account,
   display_name = empathy_account_get_display_name (account);
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
   settings = accounts_dialog_model_get_selected_settings (dialog);
+  if (settings == NULL)
+    return;
+
   selected_account = empathy_account_settings_get_account (settings);
 
   if (accounts_dialog_get_account_iter (dialog, account, &iter))
@@ -1274,8 +1273,9 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
       TP_CONNECTION_STATUS_DISCONNECTED,
       dialog);
 
-  g_signal_connect (account, "notify::display-name",
-      G_CALLBACK (accounts_dialog_account_display_name_changed_cb), dialog);
+  empathy_signal_connect_weak (account, "notify::display-name",
+      G_CALLBACK (accounts_dialog_account_display_name_changed_cb),
+      G_OBJECT (dialog));
 
   g_object_unref (settings);
 }
@@ -1330,8 +1330,6 @@ accounts_dialog_account_removed_cb (EmpathyAccountManager *manager,
 
   if (accounts_dialog_get_account_iter (dialog, account, &iter))
     {
-      g_signal_handlers_disconnect_by_func (account,
-          accounts_dialog_account_display_name_changed_cb, dialog);
       gtk_list_store_remove (GTK_LIST_STORE (
             gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview))), &iter);
     }
@@ -1432,7 +1430,7 @@ accounts_dialog_button_create_clicked_cb (GtkWidget *button,
 
   g_free (str);
 
-#ifndef HAVE_NBTK
+#ifndef HAVE_MOBLIN
   if (tp_connection_manager_protocol_can_register (proto))
     {
       gboolean active;
@@ -1629,7 +1627,7 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
   GtkBuilder                   *gui;
   gchar                        *filename;
   EmpathyAccountsDialogPriv    *priv = GET_PRIV (dialog);
-#ifdef HAVE_NBTK
+#ifdef HAVE_MOBLIN
   GtkWidget                    *action_area;
 #endif
 
@@ -1665,7 +1663,7 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
 
   g_object_unref (gui);
 
-#ifdef HAVE_NBTK
+#ifdef HAVE_MOBLIN
   action_area = gtk_dialog_get_action_area (GTK_DIALOG (priv->window));
   gtk_widget_hide (action_area);
 
